from flask import Blueprint, request, jsonify, make_response
import json
from app.logs.utils import add_alert_logs_timeline, stringify_dates
from app.utils import token_required, send_message, closed_alert, get_logo_from_logotable, paginated_query
from app.logs.models import Logs, AdvFrameType, AlertLogsHistory, AlertHistory
from app.logs.schemas import alert_logs_schema
from app import db
import pytz

logs = Blueprint("logs", __name__)

@logs.route("/api/v1/add/alert-logs", methods=['PUT'])
@token_required(1, 2, 3)
def alert_logs_save(current_user):
    try:
        data = request.form.get('formData')
        if not data:
            return make_response(jsonify({'message': 'Missing formData', "status": "error"}), 400)
        alert_logs = json.loads(data)
        alert_id = alert_logs.get('alerts_id')
        if not alert_id:
            return make_response(jsonify({'message': 'Missing alert_id in formData', "status": "error"}), 400)
        alert_history = AlertLogsHistory.query.filter_by(alert_id=alert_id).first()
        if not alert_history:
            return make_response(jsonify({'message': 'Alert history not found', "status": "error"}), 404)
        history_array = json.loads(alert_history.alert_logg_array)
        new_alert = alert_history.id
        user = json.loads(current_user.full_name)
        user_name = f"{user.get('first_name', '')} {user.get('last_name', '')}".strip()
        schema_array = {
            "note": alert_logs.get('note', ''),
            "flag": alert_logs.get('flag', '')
        }
        flag = alert_logs.get('flag')
        if flag == '1':
            history_array['user_name'] = ""
            alert_type = "Double" if alert_logs.get('alert_type') == "Emergency" else "Single"
            adv_frame_types = AdvFrameType.query.filter_by(
                device_id=alert_logs.get('device_id'),
                call_action=alert_type
            ).first()
            log_alert = Logs.query.filter_by(alert_id=alert_id).first()
            if log_alert:
                log_alert.is_alert = 0
            if adv_frame_types:
                adv_frame_types.notify = 0
                adv_frame_types.flag = 1
            db.session.commit()
        elif flag == '0':
            history_array['first_name'] = user.get('first_name', '')
            history_array['last_name'] = user.get('last_name', '')
            send_message(current_user, alert_id)
        elif flag == '2':
            closed_alert(current_user, alert_id)
        alert_logs_schema.load(schema_array)
        alert_history.alert_logg_array = json.dumps(history_array)
        alert_history.flag = int(flag)
        alert_history.note = alert_logs.get('note', '')
        alert_history.user_id = current_user.id
        db.session.commit()
        add_alert_logs_timeline(new_alert, alert_id, alert_logs.get('status', ''), user_name,
                                alert_logs.get('note'), current_user.id)
        return jsonify({'message': "Request Accepted Successfully", 'status': "Success"}), 200
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {str(e)}', "status": "error"}), 500)
@logs.route("/api/v1/get/alert-logs", methods=['GET'])
@token_required(1, 2, 3)
def get_alert_logs(current_user):
    try:
        time_zone = pytz.timezone('America/Los_Angeles')
        from_date = request.args.get('fromDate')
        to_date = request.args.get('toDate')
        logs_query = AlertHistory.query.order_by(AlertHistory.updated_at.desc())
        if from_date and to_date:
            dates = stringify_dates(from_date, to_date)
            logs_query = logs_query.filter(AlertHistory.created_at.between(dates[0], dates[1]))
        alert_logs, data = paginated_query(logs_query,
            [AlertHistory.alert_log_array],
                        AlertHistory.site_id)
        alert_logs_list = []
        for alert in alert_logs:
            if alert.alert_log_array:
                alert_data=json.loads(alert.alert_log_array)
                logo_data = ("", "")
                if alert_data['logo_id']:
                    logo_data = get_logo_from_logotable(int(alert_data['logo_id']))
                alert_logs_list.append({
                    "id": alert.id,
                    "alerts_id": alert.alert_id,
                    "patient_name": alert_data['patient_name'],
                    "device_name": alert_data['device_name'] if alert_data['device_name'] else None,
                    "device_mac": alert_data['device_mac'] if alert_data['device_mac'] else None,
                    "first_name": alert_data.get('first_name', ''),
                    "last_name": alert_data.get('last_name', ''),
                    "site_name": alert_data['site_name'] if alert_data['site_name'] else None,
                    "room_name": alert_data['room_name'] if alert_data['room_name']else None,
                    "note": alert_data['note'],
                    "logo": logo_data[0],
                    "logo_content_type": logo_data[1],
                    "time_line_logs": list(reversed(json.loads(alert.alert_log_timeline))),
                    "alert_type": alert_data['alert_type'],
                    "zone_name":  alert_data['zone_name'] if alert_data['zone_name'] else None,
                    "patient_or_staff": alert.patient_or_staff_id,
                    'created_at_date_alert': alert.created_at.astimezone(time_zone).strftime('%d %b %Y'),
                    'created_at_time_alert': alert.created_at.astimezone(time_zone).strftime('%I:%M:%S %p'),
                    "updated_at": alert.updated_at.astimezone(time_zone).strftime('%d %b %Y %I:%M:%S %p')
                })
        return jsonify({'alerts': alert_logs_list, 'data': data}), 200
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', "status": "error"}), 500)

@logs.route("/api/v1/get/staff-patient-alerts-log", methods=['GET'])
@token_required(1, 2, 3)
def get_staff_alerts_log(current_user):
    try:
        time_zone = pytz.timezone('America/Los_Angeles')
        ids = json.loads(current_user.zone_id)
        if not isinstance(ids, list):
            ids = [ids]
        logs_query = (
            AlertLogsHistory.query
            .filter(AlertLogsHistory.zone_id.in_(ids))
            .order_by(AlertLogsHistory.updated_at.desc())
        )
        preference_list = json.loads(current_user.preference)
        if not preference_list:
            return jsonify({
                'alerts': [],
                'data': {
                    'total_count': 0,
                    'total_pages': 0,
                    'start_index': 0,
                    'end_index': 0,
                    'page': 0
                }
            }), 200
        if len(preference_list) == 1:
            logs_query = logs_query.filter_by(patient_or_staff_id=preference_list[0])
        alert_logs, data = paginated_query(
            logs_query,
            [],
            AlertHistory.site_id,
            True
        )
        alert_logs_list = []
        for alert in alert_logs:
            json_data = json.loads(alert.alert_logg_array)
            logo_data = ("", "")
            if json_data.get('logo_id'):
                logo_data = get_logo_from_logotable(json_data['logo_id'])
            alert_logs_list.append({
                "id": alert.id,
                "alerts_id": alert.alert_id,
                "patient_id": json_data.get('patient_id'),
                "patient_name": json_data.get('patient_name'),
                "logo": logo_data[0],
                "logo_content_type": logo_data[1],
                "device_name": json_data.get('device_name'),
                "device_mac": json_data.get('device_mac'),
                "zone_name": json_data.get('zone_name'),
                "first_name": json_data.get('first_name', ''),
                "last_name": json_data.get('last_name', ''),
                "site_id": alert.site_id,
                "device_id": json_data.get('device_id'),
                "site_name": json_data.get('site_name'),
                "room_id": json_data['room_id'],
                "room_name": json_data.get('room_name'),
                "note": alert.note,
                "time_line_logs": "",
                "alert_type": json_data['alert_type'],
                "patient_or_staff": alert.patient_or_staff_id,
                "flag": alert.flag,
                "user_id": alert.user_id,
                'created_at_date_alert': alert.created_at.astimezone(time_zone).strftime('%d %b %Y'),
                'created_at_time_alert': alert.created_at.astimezone(time_zone).strftime('%I:%M:%S %p %Z'),
                "updated_at": alert.updated_at,
            })
        return jsonify({'alerts': alert_logs_list, 'data': data}), 200

    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', "status": "error"}), 500)